#!/usr/bin/env python3
import gensim
import os
import pdb
import gensim_plus_config
from gensim_plus_config import FLAGS
import traceback
import base_interface
from base_interface import DictionaryHelperInterface

class DictionaryHelper(DictionaryHelperInterface):

    def __init__(self):
        pass

    def newDictionary(self, wordsLst2d):
        """
        des: 使用新的文本生成词典
        in: 二维数组,元素是词
        out: 词典
        """
        try:
            dct = gensim.corpora.dictionary.Dictionary(wordsLst2d)
            return dct
        except:
            traceback.print_exc()
            return FLAGS.ERROR

    def addDoc2Dct(self, wordLst2d, dct):
        """
        使用新的文本生成词典，与现有词典合并
        """
        try:
            dct.adddocuments(wordLst2d)
            return dct
        except:
            traceback.print_exc()
            return FLAGS.ERROR

    def loadDct(self, dctPath):
        try:
            dct = gensim.corpora.dictionary.Dictionary()
            return dct.load(dctPath)
        except:
            traceback.print_exc()
            return FLAGS.ERROR

    def savDct(self, dctPath, dct):
        try:
            return dct.save(dctPath)
        except:
            traceback.print_exc()
            return FLAGS.ERROR

    def char2id(self,dct,char):
        try:
            return dct.token2id[char]
        except:
            traceback.print_exc()
            return FLAGS.ERROR

    def id2char(self,dct,_id):
        try:
            return dct.get(_id)
        except:
            traceback.print_exc()
            return FLAGS.ERROR

def func_test():
    words2dLst =[["我","爱","北京","天安门"],["我","爱","万里","长城"]]
    dctHelperIns  = DictionaryHelper()
    dct = dctHelperIns.newDictionary(words2dLst)
    print("\n> 新建词典完成",dct)
    _ = dctHelperIns.savDct(os.path.join(".","dct"),dct)
    print("\n> 保存词典完成",_)
    dct = dctHelperIns.loadDct("./dct")
    #pdb.set_trace()
    print("\n> 加载词典完成",dct)
    _ = dctHelperIns.char2id(dct,"北京")
    print("\n> 词转id",_)
    _ = dctHelperIns.id2char(dct,1)
    print("\n> id转词",_)

if __name__ == "__main__":
    func_test()





#!/usr/bin/env python3
from abc import ABCMeta,abstractmethod

class Word2VecHelperInterface(metaclass=ABCMeta):
    """
    该类别,类似与java中的interface
    接口方法在子类中,必须实现
    """
    @abstractmethod
    def setFileInPath(self,dataFileInPath):
        """
        设置输入数据路径,读取数据,均来自该目录
        """
        pass

    @abstractmethod
    def setFileOutPath(self,dateFileOutPath):
        """
        设置目标输出数据路径,写入数据,到该目录
        """
        pass

    @abstractmethod
    def setDataBasePath(self, dataBaseParameter):
        """
        设置数据库参数
        """
        pass

    @abstractmethod
    def trainWord2VecModel(self, paraDct):
        """
        训练一个词向量模型
        """
        pass


class DictionaryHelperInterface(metaclass=ABCMeta):
    """
    该类别,类似与java中的interface
    接口方法在子类中,必须实现
    """
    @abstractmethod
    def newDictionary(self, wordsLst2d):
        """
        新建词典
        """
        pass

    @abstractmethod
    def addDoc2Dct(self, wordLst2d, dct):
        """
        添加文本以补充新的词进入词典
        """
        pass

    @abstractmethod
    def loadDct(self, dctPath):
        """
        加载词典
        """
        pass

    @abstractmethod
    def savDct(self, dctPath, dct):
        """
        保存词典
        """
        pass

    @abstractmethod
    def char2id(self,dct,char):
        """
        字转id
        """

    @abstractmethod
    def id2char(self,dct,char):
        """
        保存词典
        """
        pass

class TfIdfHelperInterface(metaclass=ABCMeta):

    @abstractmethod
    def init_tfidf_model(self,dictionary,words2d):
        """
        新建tfidf模型
        """
        pass

    @abstractmethod
    def filter_tfidf(self,lst,percent):
        pass



#!/usr/bin/env python3
import os
import traceback
import pdb
import gensim
import gensim_plus_config
from gensim_plus_config import FLAGS
import jieba
import jieba.posseg
import re
import base_interface
from base_interface import Word2VecHelperInterface
import numpy as np

class Word2VecHelper(Word2VecHelperInterface):

    def __init__(self):
        pass
        print("\n> 实例化一个新的 Word2VecHelper")

    """
    def init_word_flag(self):
        self.nous = self.FLAGS.nous.split(",")
        self.adj = self.FLAGS.adj.split(",")
        self.pj = self.FLAGS.pj.split(",")
        self.verb = self.FLAGS.verb.split(",")
    """

    def setFileInPath(self,dataFileInPath):
        self.dataFileInPath = dataFileInPath

    def setFileOutPath(self,dataFileOutPath):
        self.dataFileOutPath = dataFileOutPath

    def setDataBasePath(self, dataBaseParameter):
        """
        dataBaseParameter是一个dict类型的变量用于初始化mysql的连接
        """
        self.dataBaseParameter = dataBaseParameter

    def test_trainWord2VecModel(self):
        doc = ["中华人民共和国中央人民政府今天成立了","富强民主文明和谐,自由平等公正法制,爱国敬业明理诚信"]
        w2v_model = self.trainWord2VecModel(doc,FLAGS.word2vec_4_word,128,0)
        if FLAGS.DEBUG == "True":
            pdb.set_trace()
            print(w2v_model)
        return w2v_model

    def trainWord2VecModel(self, doc, flag=FLAGS.word2vec_4_word, num_class=128,min_count=5):
        """
        docin:一维数组,元素为文本

        ex of in:['今天上午','明天下午']
        out:Word2Vec model
        """
        try:
            words_2d, flags_2d,chars_2d = [],[],[]
            if flag == FLAGS.word2vec_4_word:
                for sentence in doc:
                    word_flag_tuple = jieba.posseg.cut(sentence)
                    words = [item.word for item in word_flag_tuple]
                    words_2d.append(words)
                return gensim.models.word2vec.Word2Vec(words_2d, size=num_class, min_count=min_count)
            elif flag == FLAGS.word2vec_4_word_complex:
                for sentences in doc:
                    word_flag_tuple = jieba.posseg.cut(sentence)
                    words = [item.word for item in word_flag_tuple]
                    flags = [item.flag for item in word_flag_tuple]
                    word_flag_str = "".join(["%s/%s "%(i,j) for i,j in zip(words,flags)])
                    n_wd = re.findall("[^ ]+/(%s) "%FLAGS.nous)
                    v_wd = re.findall("[^ ]+/(%s) "%FLAGS.verb)
                    adj_wd = re.findall("[^ ]+/(%s) "%FLAGS.adj)
                    p_wd = re.findall("[^ ]+/(%s) "%FLAGS.pj)
                    words.extend(n_wd)
                    words.extend(v_wd)
                    words.extend(adj_wd)
                    words.extend(p_wd)
                    words.extend(list(sentences))
                    words.extend(flags)
                    words_2d.append(words)
                return gensim.models.word2vec.Word2Vec(words_2d, num_class=num_class, min_count=min_count)
            elif flag == FLAGS.word2vec_char:
                for sentence in doc:
                    words_2d.append(list(sentence))
                return gensim.models.word2vec.Word2Vec(words_2d, num_class=num_class, min_count=min_count)
        except:
            traceback.print_exc()
            return FLAGS.ERROR

    def loadWord2VecModel(self, word2vecModelPath):
        try:
            model = gensim.models.word2vec.Word2Vec.load(word2vecModelPath)
            return model
        except:
            traceback.print_exc()
            return FLAGS.ERROR

    def saveWord2VecModel(self, word2vecModelPath, model):
        try:
            model.save(word2vecModelPath)
        except:
            traceback.print_exc()
            return FLAGS.ERROR

    def getIdsFromModel(self,wordsLst,model,dummy,size):
        """
        in:
            wordsLst:一维数组
            model:模型
            dummy:填充字符
            size:batch长度
        out:
            一维数组,元素numpy.array([,128])
        """
        try:
            res = []
            if len(wordsLst)<size:
                wordsLst.extend([dummy]*size)
                res = wordsLst[:size]
            else:
                res = wordsLst[:size]
                print(res)
                print("\n> 有部分文本被丢弃,数量为 %s ;\n> 内容为 %s ;"%(len(wordsLst)-size,wordsLst[size:]))
            _ = np.concatenate(model[res]).reshape(size,128)
            if FLAGS.DEBUG == "True":
                pdb.set_trace()
            return _
        except:
            traceback.print_exc()
            return FLAGS.ERROR

    def test_getBatchFromModel(self):
        doc = [["自由","平等"],["富强","民主"]]
        model = self.loadWord2VecModel(os.path.join(FLAGS.current_path,"word2vec.model"))
        batch = self.getBatchFromModel(doc,model,"富强",400)
        if FLAGS.DEBUG == "True":
            pdb.set_trace()
        print(batch)

    def getBatchFromModel(self,doc,model,dummy,size):
        """
        in:
            doc:一维数组,元素string
            model:模型
            dummy:填充字符
            size:batch长度
        out:
            二维数组,元素numpy.array([,128])
        """
        try:
            res = []
            for sentence in doc:
                print(sentence)
                _ = self.getIdsFromModel(sentence,model,dummy,size)
                print(_)
                res.append(_)
            out = np.concatenate(res).reshape(len(doc),size,128)
            if FLAGS.DEBUG == "True":
                print(out,out.shape)
            return out
        except:
            traceback.print_exc()

if __name__ == "__main__":
    word2vechelper_instance = Word2VecHelper()
    model = word2vechelper_instance.test_trainWord2VecModel()
    word2vechelper_instance.saveWord2VecModel(os.path.join(FLAGS.current_path,"word2vec.model"), model)
    word2vechelper_instance.test_getBatchFromModel()




#!/usr/bin/env python3
import tensorflow as tf
import os

CURPATH = os.path.dirname(os.path.realpath(__file__))
DATAPATH = os.path.join(os.pardir, CURPATH, "data")
dictionary_file_path = os.path.join(os.pardir, CURPATH, "data", "dictionary.dct")


tf.flags.DEFINE_string("current_path", CURPATH, "current path")
tf.flags.DEFINE_string("data_path", DATAPATH, "data path")
tf.flags.DEFINE_string("dictionary_file_path", dictionary_file_path, "data path")

tf.flags.DEFINE_string("word2vec_4_word", "word2vec_4_word", "word2vec_4_word")
tf.flags.DEFINE_string("word2vec_4_word_complex", "word2vec_4_word_complex", "word2vec_4_word_complex")
tf.flags.DEFINE_string("word2vec_4_char", "word2vec_4_char", "word2vec_4_char")


tf.flags.DEFINE_string("nous", "nr,ns,nz,n", "nous word flags") # 名词
tf.flags.DEFINE_string("verb", "v,vn", "verb word falgs")       # 动词
tf.flags.DEFINE_string("adj", "a", "adj word")                  #形容词
tf.flags.DEFINE_string("pj", "p", "pj word")                    #介词

#tf.flags.DEFINE_string("DEBUG", "True", "whether is debug mode")
tf.flags.DEFINE_string("DEBUG", "False", "whether is debug mode")
tf.flags.DEFINE_string("ERROR", "except a error", "except a error")

FLAGS = tf.flags.FLAGS

print("\n",FLAGS)



#!/usr/bin/env python
# coding=utf-8


from setuptools import setup, find_packages


setup(
        name='gensim_plus',
        version=2.0,
        description=(
            'gensim在项目中的公共应用方法'
        ),
        long_description=open('README.rst').read(),
        author='秦海宁',
        author_email='qinhaining@ultrapower.com.cn',
        maintainer='秦海宁',
        maintainer_email='qinhaining@ultrapower.com.cn',
        license='BSD License',
        packages=find_packages(),
        platforms=["all"],
    url='http://www.github.com/scmsqhn/gensim-plus',
    classifiers=[
                'Development Status :: 4 - Beta',
                'Operating System :: OS Independent',
                'Intended Audience :: Developers',
                'License :: OSI Approved :: BSD License',
                'Programming Language :: Python',
                'Programming Language :: Python :: Implementation',
                'Programming Language :: Python :: 2',
                'Programming Language :: Python :: 2.7',
                'Programming Language :: Python :: 3',
                'Programming Language :: Python :: 3.4',
                'Programming Language :: Python :: 3.5',
                'Programming Language :: Python :: 3.6',
                'Topic :: Software Development :: Libraries'
    ],
)
